package org.jboss.samples.webservices;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.xml.namespace.QName;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Dispatch;
import javax.xml.ws.Service;
import javax.xml.ws.Service.Mode;
import javax.xml.ws.WebServiceException;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;


public class HelloWorldTest {

	private static final String message = "<?xml version=\"1.0\" encoding=\"utf-8\" standalone=\"yes\" ?>"
		+ "<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" "
		+ "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">"
		+ "<soap:Body><ns1:sayHello xmlns:ns1 = \"http://webservices.samples.jboss.org/\">"
		+ "<arg0>REPLACE_ME</arg0>"
		+ "</ns1:sayHello></soap:Body>"
		+ "</soap:Envelope>";
	
	private static final String wsdl = "http://localhost:8080/demo/HelloWorld?WSDL";
	
	private Dispatch<SOAPMessage> client;
	
	@Before
	public void setup() throws MalformedURLException, NoSuchAlgorithmException, KeyManagementException {
		TrustManager t = new X509TrustManager() {
			
			@Override
			public X509Certificate[] getAcceptedIssuers() {
				return new X509Certificate[0];
			}
			
			@Override
			public void checkServerTrusted(X509Certificate[] arg0, String arg1)
					throws CertificateException {
			}
			
			@Override
			public void checkClientTrusted(X509Certificate[] arg0, String arg1) {
			}
		}; 
		TrustManager[] tm = new TrustManager[] {t};
		SSLContext ctx = SSLContext.getInstance("SSL");
		ctx.init(null, tm, new SecureRandom());
		HttpsURLConnection.setDefaultSSLSocketFactory(ctx.getSocketFactory());
		HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier() {
			
			@Override
			public boolean verify(String hostname, SSLSession session) {
				return true;
			}
		});
		Service s = Service.create(new URL(wsdl), new QName("http://webservices.samples.jboss.org/", "HelloWorldService"));
		client = s.createDispatch(new QName("http://webservices.samples.jboss.org/", "HelloWorldPort"), SOAPMessage.class, Mode.MESSAGE);
		client.getRequestContext().put(BindingProvider.SOAPACTION_USE_PROPERTY, Boolean.TRUE);
		client.getRequestContext().put(BindingProvider.SOAPACTION_URI_PROPERTY, "");
	}
	
	@Test
	public void testAuth() throws SOAPException, IOException {
		client.getRequestContext().put(BindingProvider.USERNAME_PROPERTY, "tester");
		client.getRequestContext().put(BindingProvider.PASSWORD_PROPERTY, "test");
		SOAPMessage result = client.invoke(getSOAPMessage("Lukas"));
		String response = result.getSOAPBody().getFirstChild().getTextContent();
		Assert.assertEquals("Hello Lukas!", response);
	}
	
	@Test
	public void testUnauth() throws SOAPException, IOException {
		try {
			client.invoke(getSOAPMessage("Jeff"));
			Assert.fail("should fail");
		} catch (WebServiceException wse) {
			Assert.assertTrue(wse.getMessage().contains("401"));
		}
		try {
			client.getRequestContext().put(BindingProvider.USERNAME_PROPERTY, "tester");
			client.getRequestContext().put(BindingProvider.PASSWORD_PROPERTY, "testing");
			client.invoke(getSOAPMessage("Rob"));
			Assert.fail("should fail");
		} catch (WebServiceException wse) {
			Assert.assertTrue(wse.getMessage().contains("401"));
		}
	}
	
	private SOAPMessage getSOAPMessage(String user) throws IOException, SOAPException {
		return MessageFactory.newInstance().createMessage(null,
				new ByteArrayInputStream(message.replaceAll("REPLACE_ME", user).getBytes()));
	}
}
